/* image_test.cc
 * Copyright (C) 2024 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include <check.h>
#include <cmath>

#include "image.hh"

START_TEST(ham_step1)
{
    // ck_assert_int_eq( pointWithinConvexPolygon( polygon, Coord( 1.2, 1.2 ) ), true );
    //ck_assert_double_eq( round( res[0].x() * 1000 ), 0 );
    std::vector< Pixel444 > colors = { {9,8,6},
                                       {7,4,3},
                                       {12,10,9},
                                       {4,3,2},
                                       {6,7,7},
                                       {11,7,5},
                                       {9,9,9},
                                       {9,6,4},
                                       {4,5,5},
                                       {14,10,8},
                                       {10,2,2},
                                       {12,8,6},
                                       {10,10,10},
                                       {3,2,2},
                                       {8,8,9},
                                       {13,3,3} };
    Image444 img( 64, 64 );
    Palette pal;
    for ( const auto &e : colors ) {
        std::cout << "  " << e << std::endl;
        pal.add_color( e );
    }
    img.set_palette( pal );

    Pixel444 p1( 2,1,1);
    Pixel444 p2( 4,1,2);
    std::optional< Pixel444 > p3( { 6,1,2 } );
    std::optional< Pixel444 > p4( { 8,1,2 } );

    auto res = img.ham_step_lookahead( p2, p1, p3, p4, 0, 0 );
    ck_assert_int_eq( res.first.r, 2 );
    ck_assert_int_eq( res.first.g, 1 );
    ck_assert_int_eq( res.first.b, 2 );

    res = img.ham_step_lookahead( *p3, res.first, p4, {}, 2, 0 );
    ck_assert_int_eq( res.first.r, 6 );
    ck_assert_int_eq( res.first.g, 1 );
    ck_assert_int_eq( res.first.b, 2 );

    res = img.ham_step_lookahead( *p4, res.first, {}, {}, 3, 0 );
    ck_assert_int_eq( res.first.r, 8 );
    ck_assert_int_eq( res.first.g, 1 );
    ck_assert_int_eq( res.first.b, 2 );
}
END_TEST

#if 0
START_TEST(pixel_convert)
{
    for ( uint16_t c = 0; c < 256; c++ ) {
        PixelFP fp1( c, c, c, c, 256 );
        Pixel444 ip1 = fp1.convert();
        PixelFP fp2 = ip1.convert();

        std::cout << (int)c << ":" << std::endl;
        std::cout << "  " << fp1 << std::endl;
        std::cout << "  " << ip1 << std::endl;
        std::cout << "  " << fp2 << std::endl;
        std::cout << "  " << ( fp2.r * 255 ) << std::endl;
    }

    Pixel444 ip2( 15, 15, 15 );
    std::cout << "  " << ip2.convert() << std::endl;
}
END_TEST
#endif

Suite * image_suite( void )
{
    Suite *s;
    TCase *tc_image;

    s = suite_create("image");

    tc_image = tcase_create( "image" );

    tcase_add_test( tc_image, ham_step1 );
    //tcase_add_test( tc_image, pixel_convert );
    suite_add_tcase( s, tc_image );

    return s;
}

int main(void)
{
    int number_failed;
    Suite *s;
    SRunner *sr;

    ab_set_log_level( LOG_VERBOSE );

    s = image_suite();
    sr = srunner_create( s );

    srunner_run_all( sr, CK_NORMAL );
    number_failed = srunner_ntests_failed( sr );
    srunner_free( sr );
    return ( number_failed == 0 ) ? EXIT_SUCCESS : EXIT_FAILURE;
}
