/* options.hh
 * Copyright (C) 2024 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef OPTIONS_HH
#define OPTIONS_HH

#include <string>

struct ham_options {
    enum ham_lookahead {
        NO_LOOKAHEAD_FAST,
        NO_LOOKAHEAD_NORMAL,
        DIRECT_LOOKAHEAD
    };

    enum quality {
        QUALITY_FAST,
        QUALITY_NORMAL,
        QUALITY_GOOD,
        QUALITY_BEST
    };

    enum output_type {
        OUTPUT_IFF,
        OUTPUT_PPM
    };

    enum ham_lookahead lookahead_mode = DIRECT_LOOKAHEAD;
    enum quality quality_mode = QUALITY_NORMAL;

    std::string input_file;
    std::string output_file;

    bool scale = false;
    bool interlace = false;
    bool dither = true;
    bool adjust_interlace = false;

    enum output_type output_type = OUTPUT_IFF;

    bool compress_iff = false;

    bool ntsc = false;

    size_t threads = 1;
};

extern struct ham_options global_options;

#endif /* OPTIONS_HH */
