/* palette.hh
 * Copyright (C) 2024 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PALETTE_HH
#define PALETTE_HH

#include "pixel.hh"
#include <vector>
#include <optional>

class Palette {
public:
    Palette() = default;
    Palette( size_t colors );

    std::optional< Pixel444 > get_color( size_t color ) const;
    int set_color( size_t color, Pixel444 v );
    int add_color( Pixel444 v );

    std::optional< size_t > find_color( Pixel444 v ) const;
    std::optional< size_t > find_nearest_color( Pixel444 v );

    size_t size() const;

    static double color_distance( const Pixel444 &p1,
                                  const Pixel444 &p2 );

    bool operator==( const Palette &oth ) const;
private:
    void update_nearest_lookup();

    std::vector< Pixel444 > m_colors;
    std::vector< std::pair< size_t, int > > m_nearest_lookup;
};

#endif /* PALETTE_HH */
