/* pixel.cc
 * Copyright (C) 2024 Ralf Hoffmann.
 * ralf@boomerangsworld.de
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "pixel.hh"
#include "ab_logging.hh"

PixelFP::PixelFP( double _r, double _g, double _b, double _a ) :
    r( _r ),
    g( _g ),
    b( _b ),
    a( _a )
{
}

PixelFP::PixelFP( uint32_t _r, uint32_t _g, uint32_t _b, uint32_t _a, uint32_t _max_value ) :
    r( (double)_r / ( _max_value - 1 ) ),
    g( (double)_g / ( _max_value - 1 ) ),
    b( (double)_b / ( _max_value - 1 ) ),
    a( (double)_a / ( _max_value - 1 ) )
{
    if ( r < 0 ) {
        LOG_DEBUGXX( "Color underflow " << r );
        r = 0.0;
    }
    if ( r > 1 ) {
        LOG_DEBUGXX( "Color overflow " << r );
        r = 1.0;
    }
    if ( g < 0 ) {
        LOG_DEBUGXX( "Color underflow " << g );
        g = 0.0;
    }
    if ( g > 1 ) {
        LOG_DEBUGXX( "Color overflow " << g );
        g = 1.0;
    }
    if ( b < 0 ) {
        LOG_DEBUGXX( "Color underflow " << b );
        b = 0.0;
    }
    if ( b > 1 ) {
        LOG_DEBUGXX( "Color overflow " << b );
        b = 1.0;
    }
}

Pixel444 PixelFP::convert() const
{
    double cr = r < 0 ? 0 : r;
    double cg = g < 0 ? 0 : g;
    double cb = b < 0 ? 0 : b;
    uint8_t r444 = round(cr * 15);
    uint8_t g444 = round(cg * 15);
    uint8_t b444 = round(cb * 15);

    if ( r444 > 15 ) r444 = 15;
    if ( g444 > 15 ) g444 = 15;
    if ( b444 > 15 ) b444 = 15;

    return Pixel444( r444, g444, b444 );
}

Pixel444::Pixel444( uint16_t _r, uint16_t _g, uint16_t _b ) :
    r( _r ),
    g( _g ),
    b( _b )
{
}

PixelFP Pixel444::convert() const
{
    double rfp = (double)r / 15;
    double gfp = (double)g / 15;
    double bfp = (double)b / 15;

    return PixelFP( rfp, gfp, bfp );
}
